==> edna2-0.5.1.tar.gz: edna2-0.5.1/beetsplug/edna2.py <==
import flask
from flask import Flask, render_template, g, abort, url_for, request

from beets.plugins import BeetsPlugin
from beets.ui import Subcommand
import beets

app = Flask(__name__)

def get_artists():
    """
    get_artists
    
    This function will return a list of artists
    with the following format:

    [
        [
            id: artist_id,
            albums: [album_id, album_id, ...],
            artist: artist_name
        ],
        ...
    ] 
    artist_id is derived from running through all of
    the possible albums and looking for duplicated artist
    names.  Beets does not provide an artist object by
    default, so it was necessary to make this function.

    albums will contain a list of album ids corresponding
    to albums that exist in the beets library.  These ids
    are not necessarily sorted.
    """

    artists = {}
    artists_count = 0

    for a in app.config['lib'].albums():
        album = a.items().next()
        artist = album.artist

        if artist not in artists:
            artists[artist] = {"id": artists_count}
            artists[artist]["albums"] = []
            artists[artist]["artist"] = artist
            artists_count += 1
        
        artists[artist]['albums'].append(album.album_id)

    # This is for uniform appearence with the rest of
    # the bulk API, it removes the dictionaries
    # and turns them into lists
    artists = [artists[k] for k in artists]

    return artists

def proper_time(seconds):
    """
    proper_time(seconds)

    Proper time takes an argument in seconds and turns
    it into a string representation.  For example:

    >>> proper_time(60)
    '1:00'

    >>> proper_time(230.23785629)
    '3:50'

    >>> proper_time(126)
    '2:06'

    As you can see by these examples, you can feed proper_time
    either a decimal or an integer and it will come up with
    a reasonable approximation of the time.  proper_time
    will always output in the format M:SS and appends the zero
    if seconds < 10.  This function was not designed to handle
    values involving seconds > 3600 (or 60 minutes).

    >>> proper_time(3600)
    '60:00'

    >>> proper_time(3650)
    '60:50'

    Which you would expect to be 1:00:50 in human readalbe format.
    """

    seconds = int(seconds)
    minutes = seconds/60
    seconds = seconds - minutes * 60

    if seconds < 10:
        seconds = "0" + str(seconds);
    return str(minutes) + ":" + str(seconds)

def _rep(obj):
    """
    _rep(obj)

    _rep is a helper function designed to make it easier
    to translate the raw Item and Album objects provided
    by the beets library to JSON data that can be returned
    to clients.  The records (provided in the obj variable)
    are first translated into dictionaries, then have their
    fields trimmed based on what is acceptable for an Item
    or Album and what the edna2 client requires.  See those
    functions for more documentation.

    Note: 'path' for Item and 'artpath' for Album do not
    parse into correct JSON and must be deleted before
    the translation is complete.  This is done automatically
    in the _trim_* functions, but needs to happen manually
    if they are removed.
    """

    if isinstance(obj, beets.library.Item):
        out = dict(obj.record)
        if app.config['notrim']:
            del out['path']
        else:
            out = _trim_item(out)
        return out
    elif isinstance(obj, beets.library.Album):
        out = dict(obj._record)
        if app.config['notrim']:
            del out['artpath']
        else:
            out = _trim_album(out)

        # Add in the tracks
        out['tracks'] = [item.id for item in obj.items()]

        return out

def _trim_item(track):
    """
    _trim_item(track)

    This function defines accepted fields for an Item and
    passes the track object off to _trim to do the heavy
    lifting.  Accpted fields for an item are currently:

    id, title, album_id, artist, length, track, tracktotal,
    album, md_trackid, mb_albumid, mb_artistid
    """

    accepted_fields = [
        "id",
        "title",
        "album_id",
        "artist",
        "length",
        "track",
        "tracktotal",
        "album",
        "mb_trackid",
        "mb_albumid",
        "mb_artistid",
    ]

    return _trim(accepted_fields, track)

def _trim_album(album):
    """
    _trim_album(album)

    This function defines accepted fields for an Album and
    passes the track object off to _trim to do the heavy
    lifting.  Accpted fields for an album are currently:

    id, album, albumartist, tracktotal
    """

    accepted_fields = [
        "id",
        "album",
        "albumartist",
        "tracktotal"
    ]

    return _trim(accepted_fields, album) 

def _trim(accepted_fields, item):
    """
    _trim(accepted_fields, item)

    _trim is the abstract version of the _trim_*
    functions noted above.  _trim takes a dictionary called
    item and removes from that dictionary any keys that are
    not in the accpted_fields list.  This function DOES NOT
    copy the dictionary prior to removal, but rather removes
    directly from the dictionary it is passed and returns that
    dictionary.
    """

    for k in item.keys():
        if k not in accepted_fields:
            del item[k]
        # Substitute for proper time
        if k == "length":
            item[k] = proper_time(item[k])

    return item
    
def get_album_tracks(album_id):
    """
    get_album_tracks(album_id)

    This is a convinience function and only contains one list
    comprehension.  It is used twice in other functions, so it
    was factored out to keep them a little cleaner.
    """

    return [i for i in g.lib.get_album(album_id).items()]

@app.before_request
def before_request():
    g.lib = app.config['lib']

@app.route("/")
def index():
    return render_template("index.html")

@app.route('/items')
def all_items():
    """
    all_items

    PUBLIC API: /items
    This function will return a dictionary of every item 
    in the beets library in the format of:

    {
        items: [
            {
                field1,
                field2,
                ...
            },
            ...
        ]
    }

    Please view the _trim_item function for a list of what fields
    will be included in each of the items in the item list.
    """

    return flask.jsonify({"items":
                [_rep(item) for item in g.lib.items()]
           })

@app.route('/item/<int:item_id>')
def single_item(item_id):
    """
    single_item(item_id)

    PUBLIC API: /item/<item_id>
    This function will return the jsonified version of the Item
    object stored in the beets library with the give item_id.
    Please see _rep for a view of what this JSON will look like.
    """

    item = g.lib.get_item(item_id)
    return flask.jsonify(_rep(item))

@app.route('/item/<int:item_id>/file')
def item_file(item_id):
    """
    item_file(item_id)

    PUBLIC API: /item/<item_id>/file 
    This function will return the actual file giving the mimetype
    audio/mpeg using flask.send_file and the filepath given
    by the beets library with the given item_id.
    """

    item = g.lib.get_item(item_id)
    return flask.send_file(item.path, mimetype = "audio/mpeg")

@app.route('/albums')
def all_albums():
    """
    all_albums
    
    PUBLIC API: /albums
    This function will return a dictionary of every album in
    the beets library with the format:

    {
        albums: [
            {
                field1,
                field2,
                ...
            },
            ...
        ]
    }

    Please view the _trim_album function for a list of what fields
    will be included in each of the items in the albums list.
    """

    return flask.jsonify(
            {"albums":
                [_rep(album) for album in g.lib.albums()]
            })

@app.route('/album/<int:album_id>')
def single_album(album_id):
    """
    single_album(album_id)

    PUBLIC API: /album/<album_id>
    This function will return the JSON for a single item given
    by album_id.  Please see _trim_album for the exact fields
    that will be returned
    """
    return flask.jsonify(_rep(g.lib.get_album(album_id)))

@app.route('/album/<int:album_id>/art')
def album_art(album_id):
    """
    album_art(album_id)

    PUBLIC API: /album/<album_id>/art
    This function will return the album art src (if it exists) of
    the specified album_id from the path provided by the flask library.
    If the specified album does not have an art path, an empty
    string will be returned
    """

    album = g.lib.get_album(album_id)
    if album.artpath:
        return flask.send_file(album.artpath, mimetype = "image")
    else:
        return flask.send_file(app.config['default_image'], mimetype = "image") 

@app.route('/artists')
def all_artists():
    """
    all_artists

    PUBLIC API: /artists
    This function will return the jsonified output of get_artists.
    Please see that function for more information.
    """

    return flask.jsonify({"artists":app.config['artists']})

@app.route('/test')
def test():
    """
    test

    This function will render the tests (using qunit) for the
    JavaScript client portion of edna2
    """
    return render_template('test.html')

@app.route('/m3u/track/<int:track_id>.m3u')
def m3u_track(track_id):
    """
    m3u_track

    PUBLIC API: /m3u/track/<track_id>.m3u
    This function will return an m3u formatted file for
    the track given by track_id.  For example:

    m3u_track(1) yeilds
    http://edna2serveraddress.com//item/1/file

    NOTE: .m3u is a necessary extension in this case as it
    is used by desktop clients to recognize the m3u format and
    stream music from the edna2 server.  It IS REQUIRED in the
    path in order for this member of the API to be accessed.
    """
    return ("http://" + request.host + '/' + 
        url_for("item_file", item_id = track_id))

@app.route('/m3u/album/<int:album_id>.m3u')
def m3u_album(album_id):
    """
    m3u_album

    PUBLIC API: /m3u/album/<album_id>.m3u
    This function will return an m3u formatted file for
    the album given by album_id.  It will yeild the same output
    as m3u_track, but for each item_id in the album according
    to the beets library.

    NOTE: .m3u is a necessary extension in this case as it
    is used by desktop clients to recognize the m3u format and
    stream music from the edna2 server.  It IS REQUIRED in the
    path in order for this member of the API to be accessed.
    """
    # Sort by track and get the ids, then put in m3u format
    albums = [a.id for a in
                sorted(
                    get_album_tracks(album_id),
                    key = lambda album: album.track
             )]

    return "\n".join([
        "http://" + request.host + '/' +
            url_for("item_file", item_id = track_id) for track_id in albums])

@app.route('/m3u/artist/<int:artist_id>.m3u')
def m3u_artist(artist_id):
    """
    m3u_artist

    PUBLIC API: /m3u/artist/<artist_id>.m3u
    This function will return an m3u formatted file for the artist
    given by artist_id as assigned by the function get_artists.  It
    will yeild the same output as m3u_album, but for each album_id
    in the album according to the beets library.

    NOTE: .m3u is a necessary extension in this case as it
    is used by desktop clients to recognize the m3u format and
    stream music from the edna2 server.  It IS REQUIRED in the
    path in order for this member of the API to be accessed.
    """

    artists = app.config['artists'] 
    album_list = []
    ret = []

    # Find the album list associated with the artist
    for artist in artists:
        if artists[artist]["id"] == artist_id:
            album_list = artists[artist]["albums"]
            break

    # For each album in the list, get the associated tracks and then
    # list them out
    for album in album_list:

        # Make sure that the albums are in sorted order by track.  The
        # albums don't have to be sorted in any way in respect to eachother,
        # but it's nice to have sorted tracks at least
        albums = sorted(
                get_album_tracks(album),
                key = lambda album: album.track)

        # For each track in the sorted list, add them to the m3u file and
        # then return the file once we are done
        for track in albums:
            track_id = track.id
            ret.append("http://" + request.host + '/' +
                url_for("item_file", item_id = track_id))

    return "\n".join(ret)

class EdnaPlugin(BeetsPlugin):
    def __init__(self):
        super(EdnaPlugin, self).__init__()
        self.config.add({
            'host': u'',
            'port': 7000
        })

    def commands(self):
        # Initialize a new subcommand for beet
        cmd = Subcommand('edna2', help='Start the edna2 music server')
        cmd.parser.add_option('-d', '--debug', dest = "debug",
                action="store_true", default=False, help='debug mode')
        cmd.parser.add_option("--no-trim", dest = "notrim",
                action = "store_true", default=False,
                help="do not invoke the _trim_* functions")

        def func(lib, opts, args):
            # Configure default arguments for edna2 including
            # port and host
            if args:
                self.config['host'] = args.pop(0)

            if args:
                self.config['port'] = int(args.pop(0))

            # Initialize the beets library for later access
            app.config['lib'] = lib

            #Set the configuration for whether we should trim the API requests.
            #Having this option allows other clients to take a more generic
            #standpoint than the default edna2 client served by this server.
            app.config['notrim'] = opts.notrim

            # Initialize the artists object.  We want to store them in the config
            # because it will allow global access and this is relativly expensive
            # to compute especially for larger libraries.
            app.config['artists'] = get_artists()

            # The default image for albums that don't have album art will be stored
            # in this location and returned apon requets for such an album's art.
            app.config['default_image'] = "static/img/nan.png"

            # According to the flask documentation, the threaded identifier
            # will make it possible to treat each request as it's own thread.
            # This helps greatly when trying to simultaniously serve multiple
            # users and especially when trying to serve both album art and
            # a music file at the same time
            app.run(host = self.config['host'].get(str),
                    port = self.config['port'].get(int),
                    debug = opts.debug,
                    threaded = True)

        # Actually set the 'beet' command
        cmd.func = func
        return [cmd]

==> edna2-0.6.0.tar.gz: edna2-0.6.0/beetsplug/edna2.py <==
import flask
from flask import Flask, render_template, g, abort, url_for, request

from beets.plugins import BeetsPlugin
from beets.ui import Subcommand
import beets

app = Flask(__name__)

def get_artists():
    """
    get_artists
    
    This function will return a list of artists
    with the following format:

    [
        [
            id: artist_id,
            albums: [album_id, album_id, ...],
            artist: artist_name
        ],
        ...
    ] 
    artist_id is derived from running through all of
    the possible albums and looking for duplicated artist
    names.  Beets does not provide an artist object by
    default, so it was necessary to make this function.

    albums will contain a list of album ids corresponding
    to albums that exist in the beets library.  These ids
    are not necessarily sorted.
    """

    artists = {}
    artists_count = 0

    for a in app.config['lib'].albums():
        album = a.items().next()
        artist = album.artist

        if artist not in artists:
            artists[artist] = {"id": artists_count}
            artists[artist]["albums"] = []
            artists[artist]["artist"] = artist
            artists_count += 1
        
        artists[artist]['albums'].append(album.album_id)

    # This is for uniform appearence with the rest of
    # the bulk API, it removes the dictionaries
    # and turns them into lists
    artists = [artists[k] for k in artists]

    return artists

def proper_time(seconds):
    """
    proper_time(seconds)

    Proper time takes an argument in seconds and turns
    it into a string representation.  For example:

    >>> proper_time(60)
    '1:00'

    >>> proper_time(230.23785629)
    '3:50'

    >>> proper_time(126)
    '2:06'

    As you can see by these examples, you can feed proper_time
    either a decimal or an integer and it will come up with
    a reasonable approximation of the time.  proper_time
    will always output in the format M:SS and appends the zero
    if seconds < 10.  This function was not designed to handle
    values involving seconds > 3600 (or 60 minutes).

    >>> proper_time(3600)
    '60:00'

    >>> proper_time(3650)
    '60:50'

    Which you would expect to be 1:00:50 in human readalbe format.
    """

    seconds = int(seconds)
    minutes = seconds/60
    seconds = seconds - minutes * 60

    if seconds < 10:
        seconds = "0" + str(seconds);
    return str(minutes) + ":" + str(seconds)

def _rep(obj):
    """
    _rep(obj)

    _rep is a helper function designed to make it easier
    to translate the raw Item and Album objects provided
    by the beets library to JSON data that can be returned
    to clients.  The records (provided in the obj variable)
    are first translated into dictionaries, then have their
    fields trimmed based on what is acceptable for an Item
    or Album and what the edna2 client requires.  See those
    functions for more documentation.

    Note: 'path' for Item and 'artpath' for Album do not
    parse into correct JSON and must be deleted before
    the translation is complete.  This is done automatically
    in the _trim_* functions, but needs to happen manually
    if they are removed.
    """

    if isinstance(obj, beets.library.Item):
        out = dict(obj.record)
        if app.config['notrim']:
            del out['path']
        else:
            out = _trim_item(out)
        return out
    elif isinstance(obj, beets.library.Album):
        out = dict(obj._record)
        if app.config['notrim']:
            del out['artpath']
        else:
            out = _trim_album(out)

        # Add in the tracks
        out['tracks'] = [item.id for item in obj.items()]

        return out

def _trim_item(track):
    """
    _trim_item(track)

    This function defines accepted fields for an Item and
    passes the track object off to _trim to do the heavy
    lifting.  Accpted fields for an item are currently:

    id, title, album_id, artist, length, track, tracktotal,
    album, md_trackid, mb_albumid, mb_artistid
    """

    accepted_fields = [
        "id",
        "title",
        "album_id",
        "artist",
        "length",
        "track",
        "tracktotal",
        "album",
        "mb_trackid",
        "mb_albumid",
        "mb_artistid",
    ]

    return _trim(accepted_fields, track)

def _trim_album(album):
    """
    _trim_album(album)

    This function defines accepted fields for an Album and
    passes the track object off to _trim to do the heavy
    lifting.  Accpted fields for an album are currently:

    id, album, albumartist, tracktotal
    """

    accepted_fields = [
        "id",
        "album",
        "albumartist",
        "tracktotal"
    ]

    return _trim(accepted_fields, album) 

def _trim(accepted_fields, item):
    """
    _trim(accepted_fields, item)

    _trim is the abstract version of the _trim_*
    functions noted above.  _trim takes a dictionary called
    item and removes from that dictionary any keys that are
    not in the accpted_fields list.  This function DOES NOT
    copy the dictionary prior to removal, but rather removes
    directly from the dictionary it is passed and returns that
    dictionary.
    """

    for k in item.keys():
        if k not in accepted_fields:
            del item[k]
        # Substitute for proper time
        if k == "length":
            item[k] = proper_time(item[k])

    return item
    
def get_album_tracks(album_id):
    """
    get_album_tracks(album_id)

    This is a convinience function and only contains one list
    comprehension.  It is used twice in other functions, so it
    was factored out to keep them a little cleaner.
    """

    return [i for i in g.lib.get_album(album_id).items()]

@app.before_request
def before_request():
    g.lib = app.config['lib']

@app.route("/")
def index():
    return render_template("index.html")

@app.route('/items')
def all_items():
    """
    all_items

    PUBLIC API: /items
    This function will return a dictionary of every item 
    in the beets library in the format of:

    {
        items: [
            {
                field1,
                field2,
                ...
            },
            ...
        ]
    }

    Please view the _trim_item function for a list of what fields
    will be included in each of the items in the item list.
    """

    return flask.jsonify({"items":
                [_rep(item) for item in g.lib.items()]
           })

@app.route('/item/<int:item_id>')
def single_item(item_id):
    """
    single_item(item_id)

    PUBLIC API: /item/<item_id>
    This function will return the jsonified version of the Item
    object stored in the beets library with the give item_id.
    Please see _rep for a view of what this JSON will look like.
    """

    item = g.lib.get_item(item_id)
    return flask.jsonify(_rep(item))

@app.route('/item/<int:item_id>/file')
def item_file(item_id):
    """
    item_file(item_id)

    PUBLIC API: /item/<item_id>/file 
    This function will return the actual file giving the mimetype
    audio/mpeg using flask.send_file and the filepath given
    by the beets library with the given item_id.
    """

    item = g.lib.get_item(item_id)
    return flask.send_file(item.path, mimetype = "audio/mpeg")

@app.route('/albums')
def all_albums():
    """
    all_albums
    
    PUBLIC API: /albums
    This function will return a dictionary of every album in
    the beets library with the format:

    {
        albums: [
            {
                field1,
                field2,
                ...
            },
            ...
        ]
    }

    Please view the _trim_album function for a list of what fields
    will be included in each of the items in the albums list.
    """

    return flask.jsonify(
            {"albums":
                [_rep(album) for album in g.lib.albums()]
            })

@app.route('/album/<int:album_id>')
def single_album(album_id):
    """
    single_album(album_id)

    PUBLIC API: /album/<album_id>
    This function will return the JSON for a single item given
    by album_id.  Please see _trim_album for the exact fields
    that will be returned
    """
    return flask.jsonify(_rep(g.lib.get_album(album_id)))

@app.route('/album/<int:album_id>/art')
def album_art(album_id):
    """
    album_art(album_id)

    PUBLIC API: /album/<album_id>/art
    This function will return the album art src (if it exists) of
    the specified album_id from the path provided by the flask library.
    If the specified album does not have an art path, an empty
    string will be returned
    """

    album = g.lib.get_album(album_id)
    if album.artpath:
        return flask.send_file(album.artpath, mimetype = "image")
    else:
        return flask.send_file(app.config['default_image'], mimetype = "image") 

@app.route('/artists')
def all_artists():
    """
    all_artists

    PUBLIC API: /artists
    This function will return the jsonified output of get_artists.
    Please see that function for more information.
    """

    return flask.jsonify({"artists":app.config['artists']})

@app.route('/test')
def test():
    """
    test

    This function will render the tests (using qunit) for the
    JavaScript client portion of edna2
    """
    return render_template('test.html')

@app.route('/m3u/track/<int:track_id>.m3u')
def m3u_track(track_id):
    """
    m3u_track

    PUBLIC API: /m3u/track/<track_id>.m3u
    This function will return an m3u formatted file for
    the track given by track_id.  For example:

    m3u_track(1) yeilds
    http://edna2serveraddress.com//item/1/file

    NOTE: .m3u is a necessary extension in this case as it
    is used by desktop clients to recognize the m3u format and
    stream music from the edna2 server.  It IS REQUIRED in the
    path in order for this member of the API to be accessed.
    """
    return ("http://" + request.host + '/' + 
        url_for("item_file", item_id = track_id))

@app.route('/m3u/album/<int:album_id>.m3u')
def m3u_album(album_id):
    """
    m3u_album

    PUBLIC API: /m3u/album/<album_id>.m3u
    This function will return an m3u formatted file for
    the album given by album_id.  It will yeild the same output
    as m3u_track, but for each item_id in the album according
    to the beets library.

    NOTE: .m3u is a necessary extension in this case as it
    is used by desktop clients to recognize the m3u format and
    stream music from the edna2 server.  It IS REQUIRED in the
    path in order for this member of the API to be accessed.
    """
    # Sort by track and get the ids, then put in m3u format
    albums = [a.id for a in
                sorted(
                    get_album_tracks(album_id),
                    key = lambda album: album.track
             )]

    return "\n".join([
        "http://" + request.host + '/' +
            url_for("item_file", item_id = track_id) for track_id in albums])

@app.route('/m3u/artist/<int:artist_id>.m3u')
def m3u_artist(artist_id):
    """
    m3u_artist

    PUBLIC API: /m3u/artist/<artist_id>.m3u
    This function will return an m3u formatted file for the artist
    given by artist_id as assigned by the function get_artists.  It
    will yeild the same output as m3u_album, but for each album_id
    in the album according to the beets library.

    NOTE: .m3u is a necessary extension in this case as it
    is used by desktop clients to recognize the m3u format and
    stream music from the edna2 server.  It IS REQUIRED in the
    path in order for this member of the API to be accessed.
    """

    artists = app.config['artists'] 
    album_list = []
    ret = []

    # Find the album list associated with the artist
    for artist in artists:
        if artists[artist]["id"] == artist_id:
            album_list = artists[artist]["albums"]
            break

    # For each album in the list, get the associated tracks and then
    # list them out
    for album in album_list:

        # Make sure that the albums are in sorted order by track.  The
        # albums don't have to be sorted in any way in respect to eachother,
        # but it's nice to have sorted tracks at least
        albums = sorted(
                get_album_tracks(album),
                key = lambda album: album.track)

        # For each track in the sorted list, add them to the m3u file and
        # then return the file once we are done
        for track in albums:
            track_id = track.id
            ret.append("http://" + request.host + '/' +
                url_for("item_file", item_id = track_id))

    return "\n".join(ret)

class EdnaPlugin(BeetsPlugin):
    def __init__(self):
        super(EdnaPlugin, self).__init__()
        self.config.add({
            'host': u'',
            'port': 7000
        })

    def commands(self):
        # Initialize a new subcommand for beet
        cmd = Subcommand('edna2', help='Start the edna2 music server')
        cmd.parser.add_option('-d', '--debug', dest = "debug",
                action="store_true", default=False, help='debug mode')
        cmd.parser.add_option("--no-trim", dest = "notrim",
                action = "store_true", default=False,
                help="do not invoke the _trim_* functions")

        def func(lib, opts, args):
            # Configure default arguments for edna2 including
            # port and host
            if args:
                self.config['host'] = unicode(args.pop(0))

            if args:
                self.config['port'] = int(args.pop(0))

            # Initialize the beets library for later access
            app.config['lib'] = lib

            #Set the configuration for whether we should trim the API requests.
            #Having this option allows other clients to take a more generic
            #standpoint than the default edna2 client served by this server.
            app.config['notrim'] = opts.notrim

            # Initialize the artists object.  We want to store them in the config
            # because it will allow global access and this is relativly expensive
            # to compute especially for larger libraries.
            app.config['artists'] = get_artists()

            # The default image for albums that don't have album art will be stored
            # in this location and returned apon requets for such an album's art.
            app.config['default_image'] = "static/img/nan.png"

            # According to the flask documentation, the threaded identifier
            # will make it possible to treat each request as it's own thread.
            # This helps greatly when trying to simultaniously serve multiple
            # users and especially when trying to serve both album art and
            # a music file at the same time
            app.run(host = self.config['host'].get(unicode),
                    port = self.config['port'].get(int),
                    debug = opts.debug,
                    threaded = True)

        # Actually set the 'beet' command
        cmd.func = func
        return [cmd]
